#include "common/theGPIO.h"

static const char *TAG = "GPIO";
struct GPIO
{
    int pin;
    char description[20];
    int state;
};
// OLIMEX ESP-32-EVB Relays
struct GPIO Relay1 = {32, "Relay 1", 0};
struct GPIO Relay2 = {33, "Relay 2", 0};

static void print_gpio_status(struct GPIO *gpio)
{
    ESP_LOGI(TAG, "%s (Pin: %i) is %s", gpio->description, gpio->pin, gpio->state == 0 ? "Off" : "On");
}

static void change_gpio_state(struct GPIO *gpio, int state)
{
    gpio_set_direction(gpio->pin, GPIO_MODE_OUTPUT);
    gpio_set_level(gpio->pin, state);
    gpio->state = state;
    print_gpio_status(gpio);
}

static void turn_gpio_on(struct GPIO *gpio)
{
    change_gpio_state(gpio, 1);
}

static void turn_gpio_off(struct GPIO *gpio)
{
    change_gpio_state(gpio, 0);
}

static void toggle_gpio_state(struct GPIO *gpio)
{
    change_gpio_state(gpio, gpio->state == 0 ? 1 : 0);
}

static void inching_gpio_state(struct GPIO *gpio, int duration)
{
    toggle_gpio_state(gpio);
    vTaskDelay(duration / portTICK_PERIOD_MS);
    toggle_gpio_state(gpio);
}

esp_err_t relay_put_handler(httpd_req_t *req)
{
    custom_request_middleware(req);
    size_t buf_len = httpd_req_get_url_query_len(req) + 1;
    if (buf_len > 1)
    {
        char *buf = malloc(buf_len);
        if (httpd_req_get_url_query_str(req, buf, buf_len) == ESP_OK)
        {
            char relayParameter[2];
            char stateParameter[7];
            struct GPIO *relayPtr = NULL;
            if ((httpd_query_key_value(buf, "relay", relayParameter, sizeof(relayParameter)) == ESP_OK) &&
                (httpd_query_key_value(buf, "state", stateParameter, sizeof(stateParameter)) == ESP_OK))
            {
                if (strcmp(relayParameter, "1") == 0)
                    relayPtr = &Relay1;
                else if (strcmp(relayParameter, "2") == 0)
                    relayPtr = &Relay2;
                else
                {
                    custom_json_response(req, "not a valid relay", HTTPD_400);
                    return ESP_FAIL;
                }

                if (strcmp(stateParameter, "on") == 0)
                    turn_gpio_on(relayPtr);
                else if (strcmp(stateParameter, "off") == 0)
                    turn_gpio_off(relayPtr);
                else if (strcmp(stateParameter, "toggle") == 0)
                    toggle_gpio_state(relayPtr);
                else if (strcmp(stateParameter, "inch") == 0)
                    inching_gpio_state(relayPtr, 1000);
                else
                {
                    custom_json_response(req, "not a valid state", HTTPD_400);
                    return ESP_FAIL;
                }
            }
            else
            {
                custom_json_response(req, "please provide relay and state query parameter", HTTPD_400);
                return ESP_FAIL;
            }
        }
        free(buf);
        custom_json_response(req, "success", HTTPD_200);
        return ESP_OK;
    }
    custom_json_response(req, "please provide relay and state query parameter", HTTPD_400);
    return ESP_FAIL;
}
