#include "network/theFirewall.h"

static const char *TAG = "Firewall";
static struct rule_node *head = NULL;
static int amount_of_rules = 0;

typedef struct rule_node
{
    int key;
    char source[IP4ADDR_STRLEN_MAX];
    char destination[IP4ADDR_STRLEN_MAX];
    char protocol[4];
    char target[7];
    struct rule_node *next;
} rule_node_t;

static rule_node_t *add_rule_to_firewall(char *source, char *destination, char *protocol, char *target)
{
    struct rule_node *temp;
    struct rule_node *link = (struct rule_node *)malloc(sizeof(struct rule_node));
    link->key = ++amount_of_rules;
    strcpy(link->source, source);
    strcpy(link->destination, destination);
    strcpy(link->protocol, protocol);
    strcpy(link->target, target);
    if (head == NULL)
    {
        head = link;
        link->next = NULL;
        return link;
    }
    temp = head;
    while (temp->next != NULL)
    {
        temp = temp->next;
    }
    temp->next = link;
    link->next = NULL;
    return link;
}

static void prepare_firewall_json(cJSON *jsonResponse, rule_node_t *link)
{
    cJSON_AddStringToObject(jsonResponse, "status", "success");
    cJSON_AddNumberToObject(jsonResponse, "rule", link->key);
    cJSON_AddStringToObject(jsonResponse, "source", link->source);
    cJSON_AddStringToObject(jsonResponse, "destination", link->destination);
    cJSON_AddStringToObject(jsonResponse, "protocol", link->protocol);
    cJSON_AddStringToObject(jsonResponse, "target", link->target);
}

esp_err_t firewall_get_handler(httpd_req_t *req)
{
    ESP_LOGI(TAG, "Total firewall rulse: %i", amount_of_rules);
    custom_request_middleware(req);
    struct rule_node *ptr = head;
    cJSON *jsonArray = cJSON_CreateArray();
    while (ptr != NULL)
    {
        cJSON *jsonRrule = cJSON_CreateObject();
        prepare_firewall_json(jsonRrule, ptr);
        cJSON_AddItemToArray(jsonArray, jsonRrule);
        ptr = ptr->next;
    }
    const char *stringResponse = cJSON_Print(jsonArray);
    cJSON_Delete(jsonArray);
    httpd_resp_sendstr(req, stringResponse);
    free((void *)stringResponse);
    return ESP_OK;
}

esp_err_t firewall_post_handler(httpd_req_t *req)
{
    custom_request_middleware(req);
    size_t buf_len = httpd_req_get_url_query_len(req) + 1;
    if (buf_len > 1)
    {
        char *buf = malloc(buf_len);
        if (httpd_req_get_url_query_str(req, buf, buf_len) == ESP_OK)
        {
            char sourceParameter[IP4ADDR_STRLEN_MAX];
            char destinationParameter[IP4ADDR_STRLEN_MAX];
            char protocolParameter[4];
            char targetParameter[7];
            if ((httpd_query_key_value(buf, "source", sourceParameter, sizeof(sourceParameter)) == ESP_OK) &&
                (httpd_query_key_value(buf, "destination", destinationParameter, sizeof(destinationParameter)) == ESP_OK) &&
                (httpd_query_key_value(buf, "protocol", protocolParameter, sizeof(protocolParameter)) == ESP_OK) &&
                (httpd_query_key_value(buf, "target", targetParameter, sizeof(targetParameter)) == ESP_OK))
            {
                rule_node_t *link = add_rule_to_firewall(sourceParameter, destinationParameter, protocolParameter, targetParameter);
                cJSON *jsonResponse = cJSON_CreateObject();
                prepare_firewall_json(jsonResponse, link);
                const char *stringResponse = cJSON_Print(jsonResponse);
                cJSON_Delete(jsonResponse);
                httpd_resp_sendstr(req, stringResponse);
                free((void *)stringResponse);
                free(buf);
                return ESP_OK;
            }
        }
    }
    custom_json_response(req, "please provide query parameter", HTTPD_400);
    return ESP_FAIL;
}
